#include "courselib.h"
#include <cmath>
#include "TH1.h"
#include "TFile.h" 
#include "TCanvas.h"
#include <TApplication.h>
#include <TStyle.h>
#include <TROOT.h>
#include <iostream>
#include <string>
#include <iterator>
#include <iostream>
#include <cstdlib>

using namespace std;

/*
	Monte Carlo integration.
	We calculate the x-section for p p -> gamma* (Z0)--> l+l- + X
      and plot the distribution in pt and eta

	We use our own evolved pdf.

	The random number generator is RANLUX by M. Luescher.
      M. Lscher, A portable high-quality random number generator 
      for lattice field theory simulations, 
      Computer Physics Communications 79 (1994) 100  
	http://luscher.web.cern.ch/luscher/ranlux/index.html      


*/

typedef struct {
    double e;
    double px,py,pz;
    double m;
} PVEC;
#define DOTPR(p1,p2) (p1.e*p2.e-p1.px*p2.px-p1.py*p2.py-p1.pz*p2.pz)

void getpdf(double xmin, double q2,double& weightx, double& x, double& kx, double& ky, double& kt2 );

double sigma(double m2, double sh, double th)
{	
	static const double pi = 3.14159265358979323848;
	double aem=1./137.;
	double result,uh;
      result = 0;
      uh = m2 -sh -th;
// calculate dsigma/dt (qq -> gamma)
	result = 4.*pi*pi*aem/3.;
	return(result); 
}

void getnorm (double& norm);

int main(int argc,char **argv)
{
      TApplication* gMyRootApp = new TApplication("My ROOT Application", &argc, argv);
	int n1, npoints = 1e7, nacc=0;
      double normpoints=0;
 	double x1,x2;
      double x1min=0.00001, x2min=0.00001;
      double Hx1x2,Fact_gamma, mass,sh=0,th=0,q2;
      double s=4*3500.*3500.,stest;
      double kt2, kt21,kt22;
      double pt2, eta;
      double weightx1, weightx2, eq2;
      double sum0, sum00, f, ff, sigma2, error;
      double kx,ky;
      double norm;


	static const double pi = 3.14159265358979323848;
	double aem=1./137.;
	static const double gev2nb=0.3893857E+6;
    
      PVEC pin1,pin2, p_a, p_b, pout;
      
//               TH1F("label","title",nr of bins, xlow,xhigh )
      TH1F *histo100 = new TH1F("x1-pdf","x1-pdf",100, -5, 0.);
      TH1F *histo200 = new TH1F("x2-pdf","x2-pdf",100, -5, 0.);
      TH1F *histo300 = new TH1F("kt1-pdf ","kt1-pdf ",100, 0, 100.);
      TH1F *histo400 = new TH1F("kt2-pdf ","kt2-pdf",100, 0, 100.);
      TH1F *histo1 = new TH1F("x1","x1",100, -5, 0.);
      TH1F *histo2 = new TH1F("x2","x2",100, -5, 0.);
      TH1F *histo3 = new TH1F("kt1 ","kt1 ",100, 0, 10.);
      TH1F *histo4 = new TH1F("kt2 ","kt2 ",100, 0, 10.);
      TH1F *histo5 = new TH1F("pt ","pt ",100, 0, 100.);
      TH1F *histo6 = new TH1F("eta ","eta",100, -8, 8.);
      TH1F *histo7 = new TH1F("Mass ","mass",50, 60., 160.);
      
      sum0 = sum00 = 0;
      
      pin1.px = 0;
      pin1.py = 0;
      pin1.pz = sqrt(s)/2;
      pin1.e = sqrt(s)/2;
      
      pin2.px = 0;
      pin2.py = 0;
      pin2.pz = -sqrt(s)/2;
      pin2.e = sqrt(s)/2;
      
      stest = sqrt(2*DOTPR(pin1,pin2));
      
      cout << " sqrt(s) = " << stest << endl;
      eq2 = 2. *( 1. + 4. + 1. + 4.)/9. ;
      q2 = 10000;
      double mass_min = 70.;
      double mass_max = 110.;
	for ( n1 = 0; n1 < npoints; n1++ ) {
// this is using importance sampling
//		generate starting distribution in x and kt
      	getpdf(x1min, q2, weightx1, x1, kx, ky, kt2 );
//            cout << " x1 " << x1 << " kx " << kx <<" ky " << ky<<endl; 
            p_a.px = kx;
            p_a.py = ky;
            p_a.pz = sqrt(s)/2.*x1;
            p_a.e = sqrt(p_a.px*p_a.px + p_a.py*p_a.py + p_a.pz*p_a.pz);
            kt21 = p_a.px*p_a.px+p_a.py*p_a.py;
      	getpdf(x2min, q2, weightx2, x2, kx, ky, kt2 );
//            cout << " x2 " << x2 << " kx " << kx <<" ky " << ky<<endl; 
            p_b.px = kx;
            p_b.py = ky;
            p_b.pz = -sqrt(s)/2.*x2;
            p_b.e = sqrt(p_b.px*p_b.px + p_b.py*p_b.py + p_b.pz*p_b.pz);
            kt22 = p_b.px*p_b.px+p_b.py*p_b.py;
//      	cout << " kt2= " << kt2 << p_b.px*p_b.px+p_b.py*p_b.py<< endl;
		histo100->Fill(log10(x1),1/x1*weightx1);
		histo200->Fill(log10(x2),1/x2*weightx2);
		histo300->Fill(sqrt(kt21),weightx1);
		histo400->Fill(sqrt(kt22),weightx2);
            
            pout.px = p_a.px + p_b.px;
            pout.py = p_a.py + p_b.py;
            pout.pz = p_a.pz + p_b.pz;
            pout.e = p_a.e + p_b.e;
            pt2 = pout.px*pout.px + pout.py*pout.py;
            mass = sqrt(DOTPR(pout,pout));
// Fact_gamma = factor gamma*->l^+l^-
            Fact_gamma = aem/3./pi/mass/mass;
            eta = 0.5 * log(x1/x2) ;
//         if (mass > mass_min && mass < mass_max && abs(rapidity) < 3 ) {
         if (mass > mass_min && mass < mass_max ) {
//            cout << " incoming particle 1: " << p_a.px<<" "<<p_a.py <<" " <<p_a.pz <<" "<<p_a.e << endl;
//            cout << " incoming particle 2: " << p_b.px<<" "<<p_b.py <<" " <<p_b.pz <<" "<<p_b.e << endl;
//            cout << " outgoing particle: " << pout.px<<" "<<pout.py <<" " <<pout.pz <<" "<<pout.e << endl;
//            cout << " outgoing particle Mass: " << mass<< endl;
//            Fact_gamma*sigma(mass*mass, sh, th)
// 		note, pdfs are already included in weightx            
            Hx1x2 = weightx1*weightx2*eq2;
            Hx1x2 = Hx1x2*Fact_gamma*sigma(mass*mass, sh, th);
  	      Hx1x2 = Hx1x2* gev2nb;         
            	if ( x1 < 1) {
                        nacc = nacc + 1;
		            f=Hx1x2; 
//            cout<< " f= "<<f<<" "<<x1<<" "<<x2<<endl;
		            ff = f;
		            sum0 = sum0 + ff;
		            sum00 = sum00 + ff*ff; 
// weighting with 1/x0:
//	plot dxg(x)/dlogx *Jacobian, Jacobian dlogx/dx = 1/x
				histo1->Fill(log10(x1),1/x1*ff/2.3026);
				histo2->Fill(log10(x2),1/x2*ff/2.3026);
				histo3->Fill(kt21,ff);
				histo4->Fill(kt22,ff);
				histo5->Fill(sqrt(pt2),ff);
                        histo6->Fill(eta,ff);
				histo7->Fill(mass,ff);
                        
//                      cout << " mom sum " << momsum0 <<"  " << momsum <<endl;
//                      printf ("main mom sum:  %20.4f     %20.4f \n",momsum0,momsum );
		   	}
           }          
      }
//                                  

      getnorm(norm);
      cout << " norm " << norm << endl;
      normpoints = npoints/norm;
      sum0 = sum0/normpoints;
      sum00 = sum00/normpoints;
      sigma2 = sum00 - sum0*sum0 ;
      error = sqrt(sigma2/npoints) ;
      cout << " nr of events accepted: "<< nacc << endl;
      cout<< " integral for DY xsection is [nb]: " << sum0<< " +/- " << error<< endl;
//    write histogramm out to file
      TFile file("output-example15.root","RECREATE");
//general root settings 
      gROOT->Reset();
      gROOT->SetStyle("Plain");
      gStyle->SetPadTickY(1); // ticks at right side
      gStyle->SetOptStat(0); // get rid of statistics box
      TCanvas *c = new TCanvas("ctest", "" ,0, 0, 500, 500);
// divide the canvas in 1 parts in x and 1 in y
      c -> Divide(3,4);
      c -> cd(1);
      histo1 -> Draw();
      c -> cd(2);
      histo2 -> Draw();
      c -> cd(3);
      histo3 -> Draw();
      c -> cd(4);
      histo4 -> Draw();
      c -> cd(5);
      histo5 -> Draw();
      c -> cd(6);
      histo6 -> Draw();
      c -> cd(7);
      histo7 -> Draw();
      c -> cd(8);
      histo100 -> Draw();
      c -> cd(9);
      histo200 -> Draw();
      c -> cd(10);
      histo300 -> Draw();
      c -> cd(11);
      histo400 -> Draw();
      c-> Draw();
      c->WaitPrimitive();
      c-> Print("example15.pdf");
      gMyRootApp->SetReturnFromRun(true);
      histo1->Write();
      histo2->Write();
      histo3->Write();
      histo4->Write();
      histo5->Write();
      histo6->Write();
      histo7->Write();
      histo100->Write();
      histo200->Write();
      histo300->Write();
      histo400->Write();
      file.Close();
	return(0);
}
