#include "courselib.h"
#include <cmath>
#include "TH1.h"
#include "TFile.h" 
#include <iostream>
#include <string>
#include "LHAPDF/LHAPDF.h"
#include <iterator>
#include <iostream>
#include <cstdlib>

using namespace std;
using namespace LHAPDF;

/*
	Parton evolution:
      1. generate starting distribution in x and add intrisnic transverse mom.
      2. evolve sarting distribution to large values of t
            
	The random number generator is RANLUX by M. Luescher.
      M. Lscher, A portable high-quality random number generator 
      for lattice field theory simulations, 
      Computer Physics Communications 79 (1994) 100  
	http://luscher.web.cern.ch/luscher/ranlux/index.html      

*/


double gauss ( double mean, double sigma )
{	
	int n1, npoints = 12;
	double rvec[12];
      double result, r;
      result = 0.;
      r = 0;
      ranlxd(rvec,12);
	for ( n1 = 0; n1 < npoints; n1++ ) {
		r = r + rvec[n1] ;
	}
      result =mean + sigma * (r - 6);
      result = abs(result);
      return(result);
}

void get_starting_pdf(double xmin, double q2, double& weightx, double& x, double& kx, double& ky, double& kt2)
{
 		static const double pi = 3.14159265358979323848;
		double xmax = 1;
            double pdf;
            double phi;
		double rvec[2];
      	ranlxd(rvec,2);
// get x value according to g(x)\sim 1/x            
	 	x = xmin*pow(xmax/xmin,rvec[0]);
// use gluon: xg(x) = 3 (1-x)**5
            pdf = pow((1.-x),5.)*3./x ;
      	weightx = x*log(xmax/xmin) * pdf ;
// now generate instrinsic kt according to a gauss distribution            
      	kt2 = gauss(0,0.7);
//            kt2 = 0;
      	phi = 2*pi*rvec[1];            
            kx = sqrt(kt2)*cos(phi);
            ky = sqrt(kt2)*sin(phi);
//            cout << "in getpdf:  x " << x << " kx " << kx <<" ky " << ky <<endl; 
}

void sudakov(double t0, double& t)
{
//		here we calculate  from the sudakov form factor the next t > t0
 		static const double pi = 3.14159265358979323848;
		double epsilon=0.1;
            double as2pi=0.1/2./pi,Ca=3, Cf=4./3.;
            double Pint;
// for Pgg use fact = 2*Ca
            double fac=2*Ca; 
// for Pqq use fact = Cf
//            double fac=Cf; 
            double r1,t2;
		double rvec[1];
      	ranlxd(rvec,1);
//		frist do the simple case with fixed alphas and only the 1/(1-z) term of the splitting fct
		r1 = rvec[0];
		Pint=log((1.-epsilon)/epsilon);  	/* for 1/(1-z) spltiing fct */
            t2 = -log(r1)/fac/as2pi/Pint;
            t2 = t0 * exp(t2);
            t = t2;   
//            cout << " t0= "<<t0<< " t= "<<t2<< " epsilon= "<<epsilon <<endl;
            if ( t2 < t0 ) {        
            	cout << "FATAL t0 > t:  t0= "<<t0<< "t= "<<t2<< "epsilon= "<<epsilon <<endl;
            }
}

void splitting(double& z, double& weightz)
{
		double epsilon=0.1;
 		static const double pi = 3.14159265358979323848;
            double as2pi=0.1/2./pi;
            double r1,r2;
            double g0,g1,gtot;
            double zmin, zmax;
		double rvec[2];
      	ranlxd(rvec,2);
//		here we calculate  the splitting variable z

//		here we calculate  the splitting variable z for 1/z and  1/(1-z)
//		use Pgg=6 (1/z + 1/(1-z))
		g0 = 6.*as2pi * log((1-epsilon)/epsilon) ;
          	g1 = 6.*as2pi * log((1-epsilon)/epsilon) ;
          	gtot = g0 + g1 ;
          	zmin = epsilon;
          	zmax = 1.-epsilon ;
		r1 = rvec[0];
		r2 = rvec[1];
            	z = zmin*pow((zmax/zmin),r2);
            if ( r1 < g0/gtot ) 
            	z = zmin*pow((zmax/zmin),r2);
            else
//               	z = 1. - (1-zmin)*pow(((1.-zmax)/(1.-zmin)),r2);
                  z = 1. - z;
            weightz = 1.;   
            
//		approximate Pqq(z) with 1/(1-z) and generate z according to this
//		calculate according to Pqq = 4/3*as2p*(1+z**2)/(1-z)
		r1 = rvec[0];
//		z = 1. - pow(epsilon,r1);
// 		the weight is only the ratio of the (1+z**2)/(1-z)  and 1/(1-z) terms
//            weightz = (1.+z*z);

//               
//            cout << "splitting z= "<<z << " weightz = " << weightz <<endl;
		
}

void evolve_pdf(double xmin, double q2, double x0, double kx0, double ky0, double kt20,  double& weightx, double& x, double& kx, double& ky, double& kt2)
{
 		static const double pi = 3.14159265358979323848;
		double t0=1.,t1, t=0.,tcut,phi;
            double z=0, weightz;
		double epsilon=0.1;
            double ratio_splitting;
		double rvec[2];
      	ranlxd(rvec,2);
            x = x0;
            kx = kx0;
            ky = ky0;
            kt2 = kt20;
            weightx = 1. ;
            t1 = t = t0 ;
            tcut = q2;
            while (t1 < tcut ) {
      		ranlxd(rvec,2);
// 			here we do now the evolution
//			first we generate t
            	t0 = t1;
			sudakov(t0, t1) ;
//			then we generate z
			splitting(z, weightz);
// since the sudakov involves only the 1/(1-z) part of the splitting fct, we need to weight each branching
// by the ratio of the intgral of the full and approxiamte splitting fct
//			using Pgg
			ratio_splitting= 2;
// since the sudakov involves only the 1/(1-z) part of the splitting fct, we need to weight each branching
// by the ratio of the intgral of the full and approximate splitting fct
//			using Pqq
// 			ratio_splitting= (-2.*log(epsilon) - 0.5*epsilon*epsilon +3./2.*epsilon - 3./2.)/(-log(epsilon));
//                  cout << " t1 = " << t1 << " t0 = "<<t0<<endl;
//                  cout << " x = " << x << " z = "<<z<<endl;
//                  cout << " splitting ratio " << ratio_splitting <<endl;
                  if ( t1 < tcut ) { 
                     t = t1;
                     x = x*z;
                     weightx = weightx *weightz*ratio_splitting;
      	         phi = 2*pi*rvec[0];  
// 			   use here the angular ordering condition:  sqrt(t1) = qt/(1-z)
//			   and apply this also to the propagator gluons                                
                     kx = kx + sqrt(t1)*cos(phi)*(1.-z);
                     ky = ky + sqrt(t1)*sin(phi)*(1.-z);
//                     kx = kx + sqrt(t1)*cos(phi) ;
//                     ky = ky + sqrt(t1)*sin(phi);
                     kt2 = kx*kx + ky*ky ;
//                     cout << " kt = "<<sqrt(kt2) << " phi = " << phi << " t1 = " << sqrt(t1) << endl;
//                      cout << "  t= "<< t<< " x0 = " <<x0 << "x =" << x << " z =" << z <<endl;
                     }
            }
//            cout << " evolve end  t= "<< t<< " q2 " <<q2 << " x0 = " <<x0 << "x =" << x <<endl;
}


		double static momsum0=0., momsum=0.;


void getpdf(double xmin, double q2,double& weightx, double& x, double& kx, double& ky, double& kt2 )
{
		double weightx0, x0, kx0, ky0, kt20;
            double weightxf, xf, kxf, kyf, kt2f;
            weightx0= weightxf = 0;
//		generate starting distribution in x and kt
      	get_starting_pdf(xmin, q2, weightx0, x0, kx0, ky0, kt20 );
            x = x0;
            kx = kx0;
            ky = ky0;
            kt2 = kt20; 

//		now do the evolution	
		evolve_pdf(xmin, q2, x0, kx0, ky0, kt20, weightxf, xf, kxf, kyf, kt2f); 
            weightx = weightx0 *weightxf ; 
		momsum0 = momsum0+x0*weightx0 ;
		momsum = momsum+xf*weightx ;
//            cout << " getpdf: mom sum " << momsum0 <<"  " << momsum <<" " << x0 <<" " << weightx0 <<endl;

            x = xf;
            kx = kxf;
            ky = kyf;
            kt2 = kt2f; 
//            printf (" in getpdf mom sum:  %20.4f     %20.4f \n",momsum0,momsum );
}
void getnorm(double& norm)
{
		norm = momsum0/momsum ;
            printf (" in getnorm  mom sum:  %20.4f  %20.4f  %20.4f \n",momsum0,momsum,norm );
}
